import typer
from pegasusapp.train import run_train
import zipfile
import boto3
from pegasusapp.utils import destructure_s3_url
import os.path as osp
import subprocess
import os

app = typer.Typer()


@app.callback()
def callback():
    """
    Pegasus
    """


@app.command()
def train(path_to_zip: str):
    """
    Trains a pytorch lighting file
    """
    run_train(path_to_zip)


@app.command()
def configure():
    """
    Sets where you want to upload/host your models
    """
    typer.echo("Configuring")


@app.command()
def list_models():
    """
    Lists the models in your account
    """
    typer.echo("Listing")


@app.command()
def deploy():
    """
    Deploys a model
    """
    typer.echo("Deploying")


@app.command()
def delete():
    """
    Stops hosting a model
    """
    typer.echo("Deleting")


@app.command()
def pull_code_and_unzip(filepath: str):
    """
    Pulls code from the S3 repository and runs it
    """
    s3 = boto3.client("s3")
    bucket, key = destructure_s3_url(filepath, log=True)
    code_key = osp.join(key, "code.zip")
    typer.echo(code_key)
    s3.download_file(bucket, code_key, "/tmp/code.zip")

    with zipfile.ZipFile("/tmp/code.zip", "r") as zip_ref:
        zip_ref.extractall("/tmp/extracted_code")

    subprocess.run(["ls", "/tmp/extracted_code"])
    #
    # code = '''
    # #!/usr/bin/env bash
    #
    # conda activate base
    # pip install -r "$1"/requirements.txt
    # python "$1"/train.py
    # '''
    # subprocess.run(["cat", f'"{code}"', ">>", "/tmp/extracted_code/pegasus_run.sh"])
    # subprocess.run(["chmod", "+x", "/tmp/extracted_code/pegasus_run.sh"])
    # subprocess.run(["ls", "/tmp/extracted_code"])

    subprocess.run(
        ["bash", "/tmp/extracted_code/pegasus_run.sh"],
        env={**os.environ, "S3_PREFIX": filepath},
    )

    # def _build_command(command):
    #     return f"bash -c '{command}'"
    # subprocess.run(_build_command('conda init bash && conda activate base && pip install -r /tmp/extracted_code/requirements.txt'), shell=True)
    # subprocess.run(_build_command('conda init bash && conda activate base && python /tmp/extracted_code/train.py'), shell=True)

    typer.echo("Pulling code")



from urllib.parse import urlparse
from typing import Tuple
import logging
import typer

logger = logging.getLogger(__name__)


def destructure_s3_url(s3_url: str, log: bool = False) -> Tuple[str, str]:
    """
    Function to parse S3 url and returns bucket and key
    :param s3_url:
    :param log: Whether to log inputs and outputs
    :return:
    """
    o = urlparse(s3_url)
    bucket = o.netloc
    key = o.path
    key = key[1:] if key.startswith("/") else key
    if log:
        typer.echo(f"S3 URL: {s3_url}, Bucket: {bucket}, Key: {key}")
    return bucket, key



import typer
import boto3
from datetime import datetime
import os
import os.path as osp
import zipfile
from pathlib import Path
import shutil


def zipdir(path, ziph: zipfile.ZipFile):
    # ziph is zipfile handle

    from_path = str(Path(__file__).parents[1].absolute() / "pegasus_run.sh")
    ziph.write(from_path, arcname="./pegasus_run.sh")
    for root, dirs, files in os.walk(path):
        for file in files:
            ziph.write(os.path.join(root, file))


def upload_file(file_name: str, bucket: str, object_name: str = None):
    """Upload a file to an S3 bucket

    :param file_name: File to upload
    :param bucket: Bucket to upload to
    :param object_name: S3 object name. If not specified then file_name is used
    :return: True if file was uploaded, else False
    """

    # If S3 object_name was not specified, use file_name
    if object_name is None:
        object_name = file_name

    # Upload the file
    s3_client = boto3.client("s3")
    try:
        response = s3_client.upload_file(file_name, bucket, object_name)
    except Exception as e:
        typer.echo(f"Could not upload to s3 {e}")
    return True


def run_train(path_to_zip: str):
    filepath = f"ml-training/{Path(__file__).stem}/{datetime.now().timestamp()}"
    s3_code_path = f"{filepath}/code.zip"
    typer.echo(f"Compressing {path_to_zip}")

    zipf = zipfile.ZipFile("/tmp/code.zip", "w", zipfile.ZIP_DEFLATED)
    zipdir(path_to_zip, zipf)
    zipf.close()

    typer.echo(f"Uploading to s3 on path {s3_code_path}")
    upload_file(f"/tmp/code.zip", "gene", s3_code_path)

    path_for_docker = f"s3://gene/{filepath}"
    typer.echo(f"Would now call docker container with env var {path_for_docker}")

    batch = boto3.client("batch")
    command = "pip install pegasusapp && pegasus pull-code-and-unzip".split(" ")
    command.append(f"{path_for_docker}")
    batch.submit_job(
        jobName="test1",
        jobQueue="general-job-queue",
        jobDefinition="ml-training",
        containerOverrides=dict(command=command),
    )
